@@ -11,11 +11,7 @@ from optparse import make_option, OptionParser
 import django
 from django.core.exceptions import ImproperlyConfigured
 from django.core.management.color import color_style
-
-try:
-    set
-except NameError:
-    from sets import Set as set     # For Python 2.3
+from django.utils.encoding import smart_str
 
 class CommandError(Exception):
     """
@@ -28,7 +24,7 @@ class CommandError(Exception):
     result, raising this exception (with a sensible description of the
     error) is the preferred way to indicate that something has gone
     wrong in the execution of a command.
-    
+
     """
     pass
 
@@ -37,7 +33,7 @@ def handle_default_options(options):
     Include any default options that all commands should accept here
     so that ManagementUtility can handle them before searching for
     user commands.
-    
+
     """
     if options.settings:
         os.environ['DJANGO_SETTINGS_MODULE'] = options.settings
@@ -83,7 +79,7 @@ class BaseCommand(object):
     specialized methods as needed.
 
     Several attributes affect behavior at various steps along the way:
-    
+
     ``args``
         A string listing the arguments accepted by the command,
         suitable for use in help messages; e.g., a command which takes
@@ -117,7 +113,7 @@ class BaseCommand(object):
         rather than all applications' models, call
         ``self.validate(app)`` from ``handle()``, where ``app`` is the
         application's Python module.
-    
+
     """
     # Metadata about this command.
     option_list = (
@@ -147,7 +143,7 @@ class BaseCommand(object):
         Return the Django version, which should be correct for all
         built-in Django commands. User-supplied commands should
         override this method.
-        
+
         """
         return django.get_version()
 
@@ -155,7 +151,7 @@ class BaseCommand(object):
         """
         Return a brief description of how to use this command, by
         default from the attribute ``self.help``.
-        
+
         """
         usage = '%%prog %s [options] %s' % (subcommand, self.args)
         if self.help:
@@ -167,7 +163,7 @@ class BaseCommand(object):
         """
         Create and return the ``OptionParser`` which will be used to
         parse the arguments to this command.
-        
+
         """
         return OptionParser(prog=prog_name,
                             usage=self.usage(subcommand),
@@ -178,7 +174,7 @@ class BaseCommand(object):
         """
         Print the help message for this command, derived from
         ``self.usage()``.
-        
+
         """
         parser = self.create_parser(prog_name, subcommand)
         parser.print_help()
@@ -187,7 +183,7 @@ class BaseCommand(object):
         """
         Set up any environment changes requested (e.g., Python path
         and Django settings), then run this command.
-        
+
         """
         parser = self.create_parser(argv[0], argv[1])
         options, args = parser.parse_args(argv[2:])
@@ -201,7 +197,7 @@ class BaseCommand(object):
         ``self.requires_model_validation``). If the command raises a
         ``CommandError``, intercept it and print it sensibly to
         stderr.
-        
+
         """
         # Switch to English, because django-admin.py creates database content
         # like permissions, and those shouldn't contain any translations.
@@ -214,31 +210,35 @@ class BaseCommand(object):
             except ImportError, e:
                 # If settings should be available, but aren't,
                 # raise the error and quit.
-                sys.stderr.write(self.style.ERROR(str('Error: %s\n' % e)))
+                sys.stderr.write(smart_str(self.style.ERROR('Error: %s\n' % e)))
                 sys.exit(1)
         try:
+            self.stdout = options.get('stdout', sys.stdout)
+            self.stderr = options.get('stderr', sys.stderr)
             if self.requires_model_validation:
                 self.validate()
             output = self.handle(*args, **options)
             if output:
                 if self.output_transaction:
-                    # This needs to be imported here, because it relies on settings.
-                    from django.db import connection
+                    # This needs to be imported here, because it relies on
+                    # settings.
+                    from django.db import connections, DEFAULT_DB_ALIAS
+                    connection = connections[options.get('database', DEFAULT_DB_ALIAS)]
                     if connection.ops.start_transaction_sql():
-                        print self.style.SQL_KEYWORD(connection.ops.start_transaction_sql())
-                print output
+                        self.stdout.write(self.style.SQL_KEYWORD(connection.ops.start_transaction_sql()))
+                self.stdout.write(output)
                 if self.output_transaction:
-                    print self.style.SQL_KEYWORD("COMMIT;")
+                    self.stdout.write(self.style.SQL_KEYWORD("COMMIT;") + '\n')
         except CommandError, e:
-            sys.stderr.write(self.style.ERROR(str('Error: %s\n' % e)))
+            self.stderr.write(smart_str(self.style.ERROR('Error: %s\n' % e)))
             sys.exit(1)
 
     def validate(self, app=None, display_num_errors=False):
         """
         Validates the given app, raising CommandError for any errors.
-        
+
         If app is None, then this will validate all installed apps.
-        
+
         """
         from django.core.management.validation import get_validation_errors
         try:
@@ -252,13 +252,13 @@ class BaseCommand(object):
             error_text = s.read()
             raise CommandError("One or more models did not validate:\n%s" % error_text)
         if display_num_errors:
-            print "%s error%s found" % (num_errors, num_errors != 1 and 's' or '')
+            self.stdout.write("%s error%s found\n" % (num_errors, num_errors != 1 and 's' or ''))
 
     def handle(self, *args, **options):
         """
         The actual logic of the command. Subclasses must implement
         this method.
-        
+
         """
         raise NotImplementedError()
 
@@ -269,7 +269,7 @@ class AppCommand(BaseCommand):
 
     Rather than implementing ``handle()``, subclasses must implement
     ``handle_app()``, which will be called once for each application.
-    
+
     """
     args = '<appname appname ...>'
 
@@ -293,7 +293,7 @@ class AppCommand(BaseCommand):
         Perform the command's actions for ``app``, which will be the
         Python module corresponding to an application name given on
         the command line.
-        
+
         """
         raise NotImplementedError()
 
@@ -308,7 +308,7 @@ class LabelCommand(BaseCommand):
 
     If the arguments should be names of installed applications, use
     ``AppCommand`` instead.
-    
+
     """
     args = '<label label ...>'
     label = 'label'
@@ -328,7 +328,7 @@ class LabelCommand(BaseCommand):
         """
         Perform the command's actions for ``label``, which will be the
         string as given on the command line.
-        
+
         """
         raise NotImplementedError()
 
@@ -341,7 +341,7 @@ class NoArgsCommand(BaseCommand):
     no arguments are passed to the command.
 
     Attempting to pass arguments will raise ``CommandError``.
-    
+
     """
     args = ''
 
@@ -353,7 +353,7 @@ class NoArgsCommand(BaseCommand):
     def handle_noargs(self, **options):
         """
         Perform this command's actions.
-        
+
         """
         raise NotImplementedError()
 
@@ -394,9 +394,9 @@ def copy_helper(style, app_or_project, name, directory, other_name=''):
         relative_dir = d[len(template_dir)+1:].replace('%s_name' % app_or_project, name)
         if relative_dir:
             os.mkdir(os.path.join(top_dir, relative_dir))
-        for i, subdir in enumerate(subdirs):
+        for subdir in subdirs[:]:
             if subdir.startswith('.'):
-                del subdirs[i]
+                subdirs.remove(subdir)
         for f in files:
             if not f.endswith('.py'):
                 # Ignore .pyc, .pyo, .py.class etc, as they cause various
@@ -419,7 +419,7 @@ def _make_writeable(filename):
     """
     Make sure that the file is writeable. Useful if our source is
     read-only.
-    
+
     """
     import stat
     if sys.platform.startswith('java'):
