@@ -4,7 +4,7 @@
 This demonstrates features of the database API.
 """
 
-from django.db import models
+from django.db import models, DEFAULT_DB_ALIAS
 from django.conf import settings
 
 class Article(models.Model):
@@ -17,6 +17,10 @@ class Article(models.Model):
         return self.headline
 
 __test__ = {'API_TESTS': r"""
+# We can use .exists() to check that there are none yet
+>>> Article.objects.exists()
+False
+
 # Create a couple of Articles.
 >>> from datetime import datetime
 >>> a1 = Article(headline='Article 1', pub_date=datetime(2005, 7, 26))
@@ -33,9 +37,15 @@ __test__ = {'API_TESTS': r"""
 >>> a6.save()
 >>> a7 = Article(headline='Article 7', pub_date=datetime(2005, 7, 27))
 >>> a7.save()
+
+# There should be some now!
+>>> Article.objects.exists()
+True
 """}
 
-if settings.DATABASE_ENGINE in ('postgresql', 'postgresql_pysycopg2'):
+if settings.DATABASES[DEFAULT_DB_ALIAS]['ENGINE'] in (
+        'django.db.backends.postgresql',
+        'django.db.backends.postgresql_pysycopg2'):
     __test__['API_TESTS'] += r"""
 # text matching tests for PostgreSQL 8.3
 >>> Article.objects.filter(id__iexact='1')
@@ -94,6 +104,12 @@ Article 4
 <Article: Article 2>
 >>> Article.objects.in_bulk([3])
 {3: <Article: Article 3>}
+>>> Article.objects.in_bulk(set([3]))
+{3: <Article: Article 3>}
+>>> Article.objects.in_bulk(frozenset([3]))
+{3: <Article: Article 3>}
+>>> Article.objects.in_bulk((3,))
+{3: <Article: Article 3>}
 >>> Article.objects.in_bulk([1000])
 {}
 >>> Article.objects.in_bulk([])
@@ -279,6 +295,8 @@ DoesNotExist: Article matching query does not exist.
 []
 >>> Article.objects.none().count()
 0
+>>> Article.objects.none().update(headline="This should not take effect")
+0
 >>> [article for article in Article.objects.none().iterator()]
 []
 
@@ -391,7 +409,7 @@ FieldError: Join on field 'headline' not permitted. Did you misspell 'starts' fo
 """
 
 
-if settings.DATABASE_ENGINE != 'mysql':
+if settings.DATABASES[DEFAULT_DB_ALIAS]['ENGINE'] != 'django.db.backends.mysql':
     __test__['API_TESTS'] += r"""
 # grouping and backreferences
 >>> Article.objects.filter(headline__regex=r'b(.).*b\1')
