@@ -291,6 +291,7 @@ blank=True for the model field.  Finally, the widget should have the
 ...     form = AdminConcertForm
 ...     model = Concert
 ...     fk_name = 'main_band'
+...     can_delete = True
 
 >>> class BandAdmin(ModelAdmin):
 ...     inlines = [
@@ -918,7 +919,7 @@ ImproperlyConfigured: 'ValidationTestInline.extra' should be a integer.
 >>> validate(ValidationTestModelAdmin, ValidationTestModel)
 Traceback (most recent call last):
 ...
-ImproperlyConfigured: 'ValidationTestInline.max_num' should be a integer.
+ImproperlyConfigured: 'ValidationTestInline.max_num' should be an integer or None (default).
 
 >>> class ValidationTestInline(TabularInline):
 ...     model = ValidationTestInlineModel
